"""
Generated class for Extension. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.generator import FhirBaseModel
from fhirmodels.R4.Address import *
from fhirmodels.R4.Age import *
from fhirmodels.R4.Annotation import *
from fhirmodels.R4.Attachment import *
from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.Coding import *
from fhirmodels.R4.ContactDetail import *
from fhirmodels.R4.ContactPoint import *
from fhirmodels.R4.Contributor import *
from fhirmodels.R4.Count import *
from fhirmodels.R4.DataRequirement import *
from fhirmodels.R4.Distance import *
from fhirmodels.R4.Dosage import *
from fhirmodels.R4.Duration import *
from fhirmodels.R4.Expression import *
from fhirmodels.R4.HumanName import *
from fhirmodels.R4.Identifier import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Money import *
from fhirmodels.R4.ParameterDefinition import *
from fhirmodels.R4.Period import *
from fhirmodels.R4.Quantity import *
from fhirmodels.R4.Range import *
from fhirmodels.R4.Ratio import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.RelatedArtifact import *
from fhirmodels.R4.SampledData import *
from fhirmodels.R4.Signature import *
from fhirmodels.R4.Timing import *
from fhirmodels.R4.TriggerDefinition import *
from fhirmodels.R4.UsageContext import *


class Extension(FhirBaseModel):
    """Base StructureDefinition for Extension Type: Optional Extension Element - found in all resources.
    :param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param str url: identifies the meaning of the extension
    :param str valueBase64Binary: Value of extension
    :param bool valueBoolean: Value of extension
    :param str valueCanonical: Value of extension
    :param str valueCode: Value of extension
    :param str valueDate: Value of extension
    :param str valueDateTime: Value of extension
    :param float valueDecimal: Value of extension
    :param str valueId: Value of extension
    :param str valueInstant: Value of extension
    :param int valueInteger: Value of extension
    :param str valueMarkdown: Value of extension
    :param str valueOid: Value of extension
    :param int valuePositiveInt: Value of extension
    :param str valueString: Value of extension
    :param str valueTime: Value of extension
    :param int valueUnsignedInt: Value of extension
    :param str valueUri: Value of extension
    :param str valueUrl: Value of extension
    :param str valueUuid: Value of extension
    :param Address valueAddress: Value of extension
    :param Age valueAge: Value of extension
    :param Annotation valueAnnotation: Value of extension
    :param Attachment valueAttachment: Value of extension
    :param CodeableConcept valueCodeableConcept: Value of extension
    :param Coding valueCoding: Value of extension
    :param ContactPoint valueContactPoint: Value of extension
    :param Count valueCount: Value of extension
    :param Distance valueDistance: Value of extension
    :param Duration valueDuration: Value of extension
    :param HumanName valueHumanName: Value of extension
    :param Identifier valueIdentifier: Value of extension
    :param Money valueMoney: Value of extension
    :param Period valuePeriod: Value of extension
    :param Quantity valueQuantity: Value of extension
    :param Range valueRange: Value of extension
    :param Ratio valueRatio: Value of extension
    :param Reference valueReference: Value of extension
    :param SampledData valueSampledData: Value of extension
    :param Signature valueSignature: Value of extension
    :param Timing valueTiming: Value of extension
    :param ContactDetail valueContactDetail: Value of extension
    :param Contributor valueContributor: Value of extension
    :param DataRequirement valueDataRequirement: Value of extension
    :param Expression valueExpression: Value of extension
    :param ParameterDefinition valueParameterDefinition: Value of extension
    :param RelatedArtifact valueRelatedArtifact: Value of extension
    :param TriggerDefinition valueTriggerDefinition: Value of extension
    :param UsageContext valueUsageContext: Value of extension
    :param Dosage valueDosage: Value of extension
    :param Meta valueMeta: Value of extension
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "valueAddress": {"class_name": "Address", "is_contained": False},
        "valueAge": {"class_name": "Age", "is_contained": False},
        "valueAnnotation": {"class_name": "Annotation", "is_contained": False},
        "valueAttachment": {"class_name": "Attachment", "is_contained": False},
        "valueCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "valueCoding": {"class_name": "Coding", "is_contained": False},
        "valueContactPoint": {"class_name": "ContactPoint", "is_contained": False},
        "valueCount": {"class_name": "Count", "is_contained": False},
        "valueDistance": {"class_name": "Distance", "is_contained": False},
        "valueDuration": {"class_name": "Duration", "is_contained": False},
        "valueHumanName": {"class_name": "HumanName", "is_contained": False},
        "valueIdentifier": {"class_name": "Identifier", "is_contained": False},
        "valueMoney": {"class_name": "Money", "is_contained": False},
        "valuePeriod": {"class_name": "Period", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueRange": {"class_name": "Range", "is_contained": False},
        "valueRatio": {"class_name": "Ratio", "is_contained": False},
        "valueReference": {"class_name": "Reference", "is_contained": False},
        "valueSampledData": {"class_name": "SampledData", "is_contained": False},
        "valueSignature": {"class_name": "Signature", "is_contained": False},
        "valueTiming": {"class_name": "Timing", "is_contained": False},
        "valueContactDetail": {"class_name": "ContactDetail", "is_contained": False},
        "valueContributor": {"class_name": "Contributor", "is_contained": False},
        "valueDataRequirement": {
            "class_name": "DataRequirement",
            "is_contained": False,
        },
        "valueExpression": {"class_name": "Expression", "is_contained": False},
        "valueParameterDefinition": {
            "class_name": "ParameterDefinition",
            "is_contained": False,
        },
        "valueRelatedArtifact": {
            "class_name": "RelatedArtifact",
            "is_contained": False,
        },
        "valueTriggerDefinition": {
            "class_name": "TriggerDefinition",
            "is_contained": False,
        },
        "valueUsageContext": {"class_name": "UsageContext", "is_contained": False},
        "valueDosage": {"class_name": "Dosage", "is_contained": False},
        "valueMeta": {"class_name": "Meta", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        url: "str" = None,
        valueBase64Binary: "str" = None,
        valueBoolean: "bool" = None,
        valueCanonical: "str" = None,
        valueCode: "str" = None,
        valueDate: "str" = None,
        valueDateTime: "str" = None,
        valueDecimal: "float" = None,
        valueId: "str" = None,
        valueInstant: "str" = None,
        valueInteger: "int" = None,
        valueMarkdown: "str" = None,
        valueOid: "str" = None,
        valuePositiveInt: "int" = None,
        valueString: "str" = None,
        valueTime: "str" = None,
        valueUnsignedInt: "int" = None,
        valueUri: "str" = None,
        valueUrl: "str" = None,
        valueUuid: "str" = None,
        valueAddress: "Address" = None,
        valueAge: "Age" = None,
        valueAnnotation: "Annotation" = None,
        valueAttachment: "Attachment" = None,
        valueCodeableConcept: "CodeableConcept" = None,
        valueCoding: "Coding" = None,
        valueContactPoint: "ContactPoint" = None,
        valueCount: "Count" = None,
        valueDistance: "Distance" = None,
        valueDuration: "Duration" = None,
        valueHumanName: "HumanName" = None,
        valueIdentifier: "Identifier" = None,
        valueMoney: "Money" = None,
        valuePeriod: "Period" = None,
        valueQuantity: "Quantity" = None,
        valueRange: "Range" = None,
        valueRatio: "Ratio" = None,
        valueReference: "Reference" = None,
        valueSampledData: "SampledData" = None,
        valueSignature: "Signature" = None,
        valueTiming: "Timing" = None,
        valueContactDetail: "ContactDetail" = None,
        valueContributor: "Contributor" = None,
        valueDataRequirement: "DataRequirement" = None,
        valueExpression: "Expression" = None,
        valueParameterDefinition: "ParameterDefinition" = None,
        valueRelatedArtifact: "RelatedArtifact" = None,
        valueTriggerDefinition: "TriggerDefinition" = None,
        valueUsageContext: "UsageContext" = None,
        valueDosage: "Dosage" = None,
        valueMeta: "Meta" = None,
    ):

        self.id = id
        self.extension = extension or []
        self.url = url
        self.valueBase64Binary = valueBase64Binary
        self.valueBoolean = valueBoolean
        self.valueCanonical = valueCanonical
        self.valueCode = valueCode
        self.valueDate = valueDate
        self.valueDateTime = valueDateTime
        self.valueDecimal = valueDecimal
        self.valueId = valueId
        self.valueInstant = valueInstant
        self.valueInteger = valueInteger
        self.valueMarkdown = valueMarkdown
        self.valueOid = valueOid
        self.valuePositiveInt = valuePositiveInt
        self.valueString = valueString
        self.valueTime = valueTime
        self.valueUnsignedInt = valueUnsignedInt
        self.valueUri = valueUri
        self.valueUrl = valueUrl
        self.valueUuid = valueUuid
        self.valueAddress = valueAddress
        self.valueAge = valueAge
        self.valueAnnotation = valueAnnotation
        self.valueAttachment = valueAttachment
        self.valueCodeableConcept = valueCodeableConcept
        self.valueCoding = valueCoding
        self.valueContactPoint = valueContactPoint
        self.valueCount = valueCount
        self.valueDistance = valueDistance
        self.valueDuration = valueDuration
        self.valueHumanName = valueHumanName
        self.valueIdentifier = valueIdentifier
        self.valueMoney = valueMoney
        self.valuePeriod = valuePeriod
        self.valueQuantity = valueQuantity
        self.valueRange = valueRange
        self.valueRatio = valueRatio
        self.valueReference = valueReference
        self.valueSampledData = valueSampledData
        self.valueSignature = valueSignature
        self.valueTiming = valueTiming
        self.valueContactDetail = valueContactDetail
        self.valueContributor = valueContributor
        self.valueDataRequirement = valueDataRequirement
        self.valueExpression = valueExpression
        self.valueParameterDefinition = valueParameterDefinition
        self.valueRelatedArtifact = valueRelatedArtifact
        self.valueTriggerDefinition = valueTriggerDefinition
        self.valueUsageContext = valueUsageContext
        self.valueDosage = valueDosage
        self.valueMeta = valueMeta

    @classmethod
    def from_dict(cls, data: dict) -> "Extension":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Extension":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for VerificationResult. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.R4.BackboneElement import *
from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.DomainResource import *
from fhirmodels.R4.Extension import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Narrative import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.Resource import *
from fhirmodels.R4.Signature import *
from fhirmodels.R4.Timing import *


class PrimarySource(FhirBaseModel):
    """Information about the primary source(s) involved in validation.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference who: Reference to the primary source
    :param CodeableConcept type: Type of primary source (License Board; Primary Education; Continuing Education; Postal Service; Relationship owner; Registration Authority; legal source; issuing source; authoritative source)
    :param CodeableConcept communicationMethod: Method for exchanging information with the primary source
    :param CodeableConcept validationStatus: successful | failed | unknown
    :param str validationDate: When the target was validated against the primary source
    :param CodeableConcept canPushUpdates: yes | no | undetermined
    :param CodeableConcept pushTypeAvailable: specific | any | source
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "who": {"class_name": "Reference", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "communicationMethod": {"class_name": "CodeableConcept", "is_contained": False},
        "validationStatus": {"class_name": "CodeableConcept", "is_contained": False},
        "canPushUpdates": {"class_name": "CodeableConcept", "is_contained": False},
        "pushTypeAvailable": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        who: "Reference" = None,
        type: list["CodeableConcept"] = None,
        communicationMethod: list["CodeableConcept"] = None,
        validationStatus: "CodeableConcept" = None,
        validationDate: "str" = None,
        canPushUpdates: "CodeableConcept" = None,
        pushTypeAvailable: list["CodeableConcept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.who = who
        self.type = type or []
        self.communicationMethod = communicationMethod or []
        self.validationStatus = validationStatus
        self.validationDate = validationDate
        self.canPushUpdates = canPushUpdates
        self.pushTypeAvailable = pushTypeAvailable or []

    @classmethod
    def from_dict(cls, data: dict) -> "VerificationResult":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "VerificationResult":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Attestation(FhirBaseModel):
    """Information about the entity attesting to information.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference who: The individual or organization attesting to information
    :param Reference onBehalfOf: When the who is asserting on behalf of another (organization or individual)
    :param CodeableConcept communicationMethod: The method by which attested information was submitted/retrieved
    :param str date: The date the information was attested to
    :param str sourceIdentityCertificate: A digital identity certificate associated with the attestation source
    :param str proxyIdentityCertificate: A digital identity certificate associated with the proxy entity submitting attested information on behalf of the attestation source
    :param Signature proxySignature: Proxy signature
    :param Signature sourceSignature: Attester signature
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "who": {"class_name": "Reference", "is_contained": False},
        "onBehalfOf": {"class_name": "Reference", "is_contained": False},
        "communicationMethod": {"class_name": "CodeableConcept", "is_contained": False},
        "proxySignature": {"class_name": "Signature", "is_contained": False},
        "sourceSignature": {"class_name": "Signature", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        who: "Reference" = None,
        onBehalfOf: "Reference" = None,
        communicationMethod: "CodeableConcept" = None,
        date: "str" = None,
        sourceIdentityCertificate: "str" = None,
        proxyIdentityCertificate: "str" = None,
        proxySignature: "Signature" = None,
        sourceSignature: "Signature" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.who = who
        self.onBehalfOf = onBehalfOf
        self.communicationMethod = communicationMethod
        self.date = date
        self.sourceIdentityCertificate = sourceIdentityCertificate
        self.proxyIdentityCertificate = proxyIdentityCertificate
        self.proxySignature = proxySignature
        self.sourceSignature = sourceSignature

    @classmethod
    def from_dict(cls, data: dict) -> "VerificationResult":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "VerificationResult":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Validator(FhirBaseModel):
    """Information about the entity validating information.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference organization: Reference to the organization validating information
    :param str identityCertificate: A digital identity certificate associated with the validator
    :param Signature attestationSignature: Validator signature
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "organization": {"class_name": "Reference", "is_contained": False},
        "attestationSignature": {"class_name": "Signature", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        organization: "Reference" = None,
        identityCertificate: "str" = None,
        attestationSignature: "Signature" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.organization = organization
        self.identityCertificate = identityCertificate
        self.attestationSignature = attestationSignature

    @classmethod
    def from_dict(cls, data: dict) -> "VerificationResult":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "VerificationResult":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class VerificationResult(DomainResource):
    """Describes validation requirements, source(s), status and dates for one or more elements.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Reference target: A resource that was validated
    :param str targetLocation: The fhirpath location(s) within the resource that was validated
    :param CodeableConcept need: none | initial | periodic
    :param str status: attested | validated | in-process | req-revalid | val-fail | reval-fail
    :param str statusDate: When the validation status was updated
    :param CodeableConcept validationType: nothing | primary | multiple
    :param CodeableConcept validationProcess: The primary process by which the target is validated (edit check; value set; primary source; multiple sources; standalone; in context)
    :param Timing frequency: Frequency of revalidation
    :param str lastPerformed: The date/time validation was last completed (including failed validations)
    :param str nextScheduled: The date when target is next validated, if appropriate
    :param CodeableConcept failureAction: fatal | warn | rec-only | none
    :param PrimarySource primarySource: Information about the primary source(s) involved in validation
    :param Attestation attestation: Information about the entity attesting to information
    :param Validator validator: Information about the entity validating information
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "target": {"class_name": "Reference", "is_contained": False},
        "need": {"class_name": "CodeableConcept", "is_contained": False},
        "validationType": {"class_name": "CodeableConcept", "is_contained": False},
        "validationProcess": {"class_name": "CodeableConcept", "is_contained": False},
        "frequency": {"class_name": "Timing", "is_contained": False},
        "failureAction": {"class_name": "CodeableConcept", "is_contained": False},
        "primarySource": {"class_name": "PrimarySource", "is_contained": True},
        "attestation": {"class_name": "Attestation", "is_contained": True},
        "validator": {"class_name": "Validator", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        target: list["Reference"] = None,
        targetLocation: list["str"] = None,
        need: "CodeableConcept" = None,
        status: "str" = None,
        statusDate: "str" = None,
        validationType: "CodeableConcept" = None,
        validationProcess: list["CodeableConcept"] = None,
        frequency: "Timing" = None,
        lastPerformed: "str" = None,
        nextScheduled: "str" = None,
        failureAction: "CodeableConcept" = None,
        primarySource: list["PrimarySource"] = None,
        attestation: "Attestation" = None,
        validator: list["Validator"] = None,
    ):

        self.resourceType = "VerificationResult"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.target = target or []
        self.targetLocation = targetLocation or []
        self.need = need
        self.status = status
        self.statusDate = statusDate
        self.validationType = validationType
        self.validationProcess = validationProcess or []
        self.frequency = frequency
        self.lastPerformed = lastPerformed
        self.nextScheduled = nextScheduled
        self.failureAction = failureAction
        self.primarySource = primarySource or []
        self.attestation = attestation
        self.validator = validator or []

    @classmethod
    def from_dict(cls, data: dict) -> "VerificationResult":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "VerificationResult":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for GuidanceResponse. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.Annotation import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.DataRequirement import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Reference import *
from fhirmodels.R4B.Resource import *


class GuidanceResponse(DomainResource):
    """A guidance response is the formal response to a guidance request, including any output parameters returned by the evaluation, as well as the description of any proposed actions to be taken.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier requestIdentifier: The identifier of the request associated with this response, if any
    :param Identifier identifier: Business identifier
    :param str moduleUri: What guidance was requested
    :param str moduleCanonical: What guidance was requested
    :param CodeableConcept moduleCodeableConcept: What guidance was requested
    :param str status: success | data-requested | data-required | in-progress | failure | entered-in-error
    :param Reference subject: Patient the request was performed for
    :param Reference encounter: Encounter during which the response was returned
    :param str occurrenceDateTime: When the guidance response was processed
    :param Reference performer: Device returning the guidance
    :param CodeableConcept reasonCode: Why guidance is needed
    :param Reference reasonReference: Why guidance is needed
    :param Annotation note: Additional notes about the response
    :param Reference evaluationMessage: Messages resulting from the evaluation of the artifact or artifacts
    :param Reference outputParameters: The output parameters of the evaluation, if any
    :param Reference result: Proposed actions, if any
    :param DataRequirement dataRequirement: Additional required data
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "requestIdentifier": {"class_name": "Identifier", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "moduleCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "subject": {"class_name": "Reference", "is_contained": False},
        "encounter": {"class_name": "Reference", "is_contained": False},
        "performer": {"class_name": "Reference", "is_contained": False},
        "reasonCode": {"class_name": "CodeableConcept", "is_contained": False},
        "reasonReference": {"class_name": "Reference", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
        "evaluationMessage": {"class_name": "Reference", "is_contained": False},
        "outputParameters": {"class_name": "Reference", "is_contained": False},
        "result": {"class_name": "Reference", "is_contained": False},
        "dataRequirement": {"class_name": "DataRequirement", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        requestIdentifier: "Identifier" = None,
        identifier: list["Identifier"] = None,
        moduleUri: "str" = None,
        moduleCanonical: "str" = None,
        moduleCodeableConcept: "CodeableConcept" = None,
        status: "str" = None,
        subject: "Reference" = None,
        encounter: "Reference" = None,
        occurrenceDateTime: "str" = None,
        performer: "Reference" = None,
        reasonCode: list["CodeableConcept"] = None,
        reasonReference: list["Reference"] = None,
        note: list["Annotation"] = None,
        evaluationMessage: list["Reference"] = None,
        outputParameters: "Reference" = None,
        result: "Reference" = None,
        dataRequirement: list["DataRequirement"] = None,
    ):

        self.resourceType = "GuidanceResponse"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.requestIdentifier = requestIdentifier
        self.identifier = identifier or []
        self.moduleUri = moduleUri
        self.moduleCanonical = moduleCanonical
        self.moduleCodeableConcept = moduleCodeableConcept
        self.status = status
        self.subject = subject
        self.encounter = encounter
        self.occurrenceDateTime = occurrenceDateTime
        self.performer = performer
        self.reasonCode = reasonCode or []
        self.reasonReference = reasonReference or []
        self.note = note or []
        self.evaluationMessage = evaluationMessage or []
        self.outputParameters = outputParameters
        self.result = result
        self.dataRequirement = dataRequirement or []

    @classmethod
    def from_dict(cls, data: dict) -> "GuidanceResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "GuidanceResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Specimen. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.Annotation import *
from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.CodeableReference import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Duration import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Quantity import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class Feature(FhirBaseModel):
    """A physical feature or landmark on a specimen, highlighted for context by the collector of the specimen (e.g. surgeon), that identifies the type of feature as well as its meaning (e.g. the red ink indicating the resection margin of the right lobe of the excised prostate tissue or wire loop at radiologically suspected tumor location).:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Highlighted feature
    :param str description: Information about the feature
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        description: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.description = description

    @classmethod
    def from_dict(cls, data: dict) -> "Specimen":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Specimen":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Collection(FhirBaseModel):
    """Details concerning the specimen collection.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference collector: Who collected the specimen
    :param str collectedDateTime: Collection time
    :param Period collectedPeriod: Collection time
    :param Duration duration: How long it took to collect specimen
    :param Quantity quantity: The quantity of specimen collected
    :param CodeableConcept method: Technique used to perform collection
    :param CodeableReference device: Device used to perform collection
    :param Reference procedure: The procedure that collects the specimen
    :param CodeableReference bodySite: Anatomical collection site
    :param CodeableConcept fastingStatusCodeableConcept: Whether or how long patient abstained from food and/or drink
    :param Duration fastingStatusDuration: Whether or how long patient abstained from food and/or drink
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "collector": {"class_name": "Reference", "is_contained": False},
        "collectedPeriod": {"class_name": "Period", "is_contained": False},
        "duration": {"class_name": "Duration", "is_contained": False},
        "quantity": {"class_name": "Quantity", "is_contained": False},
        "method": {"class_name": "CodeableConcept", "is_contained": False},
        "device": {"class_name": "CodeableReference", "is_contained": False},
        "procedure": {"class_name": "Reference", "is_contained": False},
        "bodySite": {"class_name": "CodeableReference", "is_contained": False},
        "fastingStatusCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "fastingStatusDuration": {"class_name": "Duration", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        collector: "Reference" = None,
        collectedDateTime: "str" = None,
        collectedPeriod: "Period" = None,
        duration: "Duration" = None,
        quantity: "Quantity" = None,
        method: "CodeableConcept" = None,
        device: "CodeableReference" = None,
        procedure: "Reference" = None,
        bodySite: "CodeableReference" = None,
        fastingStatusCodeableConcept: "CodeableConcept" = None,
        fastingStatusDuration: "Duration" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.collector = collector
        self.collectedDateTime = collectedDateTime
        self.collectedPeriod = collectedPeriod
        self.duration = duration
        self.quantity = quantity
        self.method = method
        self.device = device
        self.procedure = procedure
        self.bodySite = bodySite
        self.fastingStatusCodeableConcept = fastingStatusCodeableConcept
        self.fastingStatusDuration = fastingStatusDuration

    @classmethod
    def from_dict(cls, data: dict) -> "Specimen":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Specimen":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Processing(FhirBaseModel):
    """Details concerning processing and processing steps for the specimen.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str description: Textual description of procedure
    :param CodeableConcept method: Indicates the treatment step  applied to the specimen
    :param Reference additive: Material used in the processing step
    :param str timeDateTime: Date and time of specimen processing
    :param Period timePeriod: Date and time of specimen processing
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "method": {"class_name": "CodeableConcept", "is_contained": False},
        "additive": {"class_name": "Reference", "is_contained": False},
        "timePeriod": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        description: "str" = None,
        method: "CodeableConcept" = None,
        additive: list["Reference"] = None,
        timeDateTime: "str" = None,
        timePeriod: "Period" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.description = description
        self.method = method
        self.additive = additive or []
        self.timeDateTime = timeDateTime
        self.timePeriod = timePeriod

    @classmethod
    def from_dict(cls, data: dict) -> "Specimen":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Specimen":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Container(FhirBaseModel):
    """The container holding the specimen.  The recursive nature of containers; i.e. blood in tube in tray in rack is not addressed here.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference device: Device resource for the container
    :param Reference location: Where the container is
    :param Quantity specimenQuantity: Quantity of specimen within container
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "device": {"class_name": "Reference", "is_contained": False},
        "location": {"class_name": "Reference", "is_contained": False},
        "specimenQuantity": {"class_name": "Quantity", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        device: "Reference" = None,
        location: "Reference" = None,
        specimenQuantity: "Quantity" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.device = device
        self.location = location
        self.specimenQuantity = specimenQuantity

    @classmethod
    def from_dict(cls, data: dict) -> "Specimen":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Specimen":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Specimen(DomainResource):
    """A sample to be used for analysis.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: External Identifier
    :param Identifier accessionIdentifier: Identifier assigned by the lab
    :param str status: available | unavailable | unsatisfactory | entered-in-error
    :param CodeableConcept type: Kind of material that forms the specimen
    :param Reference subject: Where the specimen came from. This may be from patient(s), from a location (e.g., the source of an environmental sample), or a sampling of a substance, a biologically-derived product, or a device
    :param str receivedTime: The time when specimen is received by the testing laboratory
    :param Reference parent: Specimen from which this specimen originated
    :param Reference request: Why the specimen was collected
    :param str combined: grouped | pooled
    :param CodeableConcept role: The role the specimen serves
    :param Feature feature: The physical feature of a specimen
    :param Collection collection: Collection details
    :param Processing processing: Processing and processing step details
    :param Container container: Direct container of specimen (tube/slide, etc.)
    :param CodeableConcept condition: State of the specimen
    :param Annotation note: Comments
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "accessionIdentifier": {"class_name": "Identifier", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "subject": {"class_name": "Reference", "is_contained": False},
        "parent": {"class_name": "Reference", "is_contained": False},
        "request": {"class_name": "Reference", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "feature": {"class_name": "Feature", "is_contained": True},
        "collection": {"class_name": "Collection", "is_contained": True},
        "processing": {"class_name": "Processing", "is_contained": True},
        "container": {"class_name": "Container", "is_contained": True},
        "condition": {"class_name": "CodeableConcept", "is_contained": False},
        "note": {"class_name": "Annotation", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        accessionIdentifier: "Identifier" = None,
        status: "str" = None,
        type: "CodeableConcept" = None,
        subject: "Reference" = None,
        receivedTime: "str" = None,
        parent: list["Reference"] = None,
        request: list["Reference"] = None,
        combined: "str" = None,
        role: list["CodeableConcept"] = None,
        feature: list["Feature"] = None,
        collection: "Collection" = None,
        processing: list["Processing"] = None,
        container: list["Container"] = None,
        condition: list["CodeableConcept"] = None,
        note: list["Annotation"] = None,
    ):

        self.resourceType = "Specimen"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.accessionIdentifier = accessionIdentifier
        self.status = status
        self.type = type
        self.subject = subject
        self.receivedTime = receivedTime
        self.parent = parent or []
        self.request = request or []
        self.combined = combined
        self.role = role or []
        self.feature = feature or []
        self.collection = collection
        self.processing = processing or []
        self.container = container or []
        self.condition = condition or []
        self.note = note or []

    @classmethod
    def from_dict(cls, data: dict) -> "Specimen":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Specimen":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for HumanName. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.generator import FhirBaseModel
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Period import *


class HumanName(FhirBaseModel):
    """HumanName Type: A name, normally of a human, that can be used for other living entities (e.g. animals but not organizations) that have been assigned names by a human and may need the use of name parts or the need for usage information.
    :param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param str use: usual | official | temp | nickname | anonymous | old | maiden
    :param str text: Text representation of the full name
    :param str family: Family name (often called 'Surname')
    :param str given: Given names (not always 'first'). Includes middle names
    :param str prefix: Parts that come before the name
    :param str suffix: Parts that come after the name
    :param Period period: Time period when name was/is in use
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        use: "str" = None,
        text: "str" = None,
        family: "str" = None,
        given: list["str"] = None,
        prefix: list["str"] = None,
        suffix: list["str"] = None,
        period: "Period" = None,
    ):

        self.id = id
        self.extension = extension or []
        self.use = use
        self.text = text
        self.family = family
        self.given = given or []
        self.prefix = prefix or []
        self.suffix = suffix or []
        self.period = period

    @classmethod
    def from_dict(cls, data: dict) -> "HumanName":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "HumanName":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for DeviceDefinition. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.Annotation import *
from fhirmodels.R5.Attachment import *
from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.CodeableReference import *
from fhirmodels.R5.Coding import *
from fhirmodels.R5.ContactPoint import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.ProductShelfLife import *
from fhirmodels.R5.Quantity import *
from fhirmodels.R5.Range import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.RelatedArtifact import *
from fhirmodels.R5.Resource import *
from fhirmodels.R5.UsageContext import *


class MarketDistribution(FhirBaseModel):
    """Indicates where and when the device is available on the market.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Period marketPeriod: Begin and end dates for the commercial distribution of the device
    :param str subJurisdiction: National state or territory where the device is commercialized
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "marketPeriod": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        marketPeriod: "Period" = None,
        subJurisdiction: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.marketPeriod = marketPeriod
        self.subJurisdiction = subJurisdiction

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class UdiDeviceIdentifier(FhirBaseModel):
    """Unique device identifier (UDI) assigned to device label or package.  Note that the Device may include multiple udiCarriers as it either may include just the udiCarrier for the jurisdiction it is sold, or for multiple jurisdictions it could have been sold.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str deviceIdentifier: The identifier that is to be associated with every Device that references this DeviceDefintiion for the issuer and jurisdiction provided in the DeviceDefinition.udiDeviceIdentifier
    :param str issuer: The organization that assigns the identifier algorithm
    :param str jurisdiction: The jurisdiction to which the deviceIdentifier applies
    :param MarketDistribution marketDistribution: Indicates whether and when the device is available on the market
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "marketDistribution": {
            "class_name": "MarketDistribution",
            "is_contained": True,
        },
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        deviceIdentifier: "str" = None,
        issuer: "str" = None,
        jurisdiction: "str" = None,
        marketDistribution: list["MarketDistribution"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.deviceIdentifier = deviceIdentifier
        self.issuer = issuer
        self.jurisdiction = jurisdiction
        self.marketDistribution = marketDistribution or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class RegulatoryIdentifier(FhirBaseModel):
    """Identifier associated with the regulatory documentation (certificates, technical documentation, post-market surveillance documentation and reports) of a set of device models sharing the same intended purpose, risk class and essential design and manufacturing characteristics. One example is the Basic UDI-DI in Europe.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str type: basic | master | license
    :param str deviceIdentifier: The identifier itself
    :param str issuer: The organization that issued this identifier
    :param str jurisdiction: The jurisdiction to which the deviceIdentifier applies
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "str" = None,
        deviceIdentifier: "str" = None,
        issuer: "str" = None,
        jurisdiction: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.deviceIdentifier = deviceIdentifier
        self.issuer = issuer
        self.jurisdiction = jurisdiction

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class DeviceName(FhirBaseModel):
    """The name or names of the device as given by the manufacturer.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str name: A name that is used to refer to the device
    :param str type: registered-name | user-friendly-name | patient-reported-name
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        name: "str" = None,
        type: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.name = name
        self.type = type

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Classification(FhirBaseModel):
    """What kind of device or device system this is.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: A classification or risk class of the device model
    :param RelatedArtifact justification: Further information qualifying this classification of the device model
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "justification": {"class_name": "RelatedArtifact", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        justification: list["RelatedArtifact"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.justification = justification or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class ConformsTo(FhirBaseModel):
    """Identifies the standards, specifications, or formal guidances for the capabilities supported by the device. The device may be certified as conformant to these specifications e.g., communication, performance, process, measurement, or specialization standards.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept category: Describes the common type of the standard, specification, or formal guidance
    :param CodeableConcept specification: Identifies the standard, specification, or formal guidance that the device adheres to the Device Specification type
    :param str version: The specific form or variant of the standard, specification or formal guidance
    :param RelatedArtifact source: Standard, regulation, certification, or guidance website, document, or other publication, or similar, supporting the conformance
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "category": {"class_name": "CodeableConcept", "is_contained": False},
        "specification": {"class_name": "CodeableConcept", "is_contained": False},
        "source": {"class_name": "RelatedArtifact", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        category: "CodeableConcept" = None,
        specification: "CodeableConcept" = None,
        version: list["str"] = None,
        source: list["RelatedArtifact"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.category = category
        self.specification = specification
        self.version = version or []
        self.source = source or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class HasPart(FhirBaseModel):
    """A device that is part (for example a component) of the present device.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference reference: Reference to the part
    :param int count: Number of occurrences of the part
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "reference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        reference: "Reference" = None,
        count: "int" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.reference = reference
        self.count = count

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Distributor(FhirBaseModel):
    """An organization that distributes the packaged device.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str name: Distributor's human-readable name
    :param Reference organizationReference: Distributor as an Organization resource
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "organizationReference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        name: "str" = None,
        organizationReference: list["Reference"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.name = name
        self.organizationReference = organizationReference or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Packaging(FhirBaseModel):
    """Information about the packaging of the device, i.e. how the device is packaged.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Identifier identifier: Business identifier of the packaged medication
    :param CodeableConcept type: A code that defines the specific type of packaging
    :param int count: The number of items contained in the package (devices or sub-packages)
    :param Distributor distributor: An organization that distributes the packaged device
    :param UdiDeviceIdentifier udiDeviceIdentifier: Unique Device Identifier (UDI) Barcode string on the packaging
    :param Packaging packaging: Allows packages within packages
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "distributor": {"class_name": "Distributor", "is_contained": True},
        "udiDeviceIdentifier": {
            "class_name": "UdiDeviceIdentifier",
            "is_contained": True,
        },
        "packaging": {"class_name": "Packaging", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: "Identifier" = None,
        type: "CodeableConcept" = None,
        count: "int" = None,
        distributor: list["Distributor"] = None,
        udiDeviceIdentifier: list["UdiDeviceIdentifier"] = None,
        packaging: list["Packaging"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier
        self.type = type
        self.count = count
        self.distributor = distributor or []
        self.udiDeviceIdentifier = udiDeviceIdentifier or []
        self.packaging = packaging or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Version(FhirBaseModel):
    """The version of the device or software.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: The type of the device version, e.g. manufacturer, approved, internal
    :param Identifier component: The hardware or software module of the device to which the version applies
    :param str value: The version text
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "component": {"class_name": "Identifier", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        component: "Identifier" = None,
        value: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.component = component
        self.value = value

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Property(FhirBaseModel):
    """Static or essentially fixed characteristics or features of this kind of device that are otherwise not captured in more specific attributes, e.g., time or timing attributes, resolution, accuracy, and physical attributes.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Code that specifies the property being represented
    :param Quantity valueQuantity: Value of the property
    :param CodeableConcept valueCodeableConcept: Value of the property
    :param str valueString: Value of the property
    :param bool valueBoolean: Value of the property
    :param int valueInteger: Value of the property
    :param Range valueRange: Value of the property
    :param Attachment valueAttachment: Value of the property
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "valueRange": {"class_name": "Range", "is_contained": False},
        "valueAttachment": {"class_name": "Attachment", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        valueQuantity: "Quantity" = None,
        valueCodeableConcept: "CodeableConcept" = None,
        valueString: "str" = None,
        valueBoolean: "bool" = None,
        valueInteger: "int" = None,
        valueRange: "Range" = None,
        valueAttachment: "Attachment" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.valueQuantity = valueQuantity
        self.valueCodeableConcept = valueCodeableConcept
        self.valueString = valueString
        self.valueBoolean = valueBoolean
        self.valueInteger = valueInteger
        self.valueRange = valueRange
        self.valueAttachment = valueAttachment

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Link(FhirBaseModel):
    """An associated device, attached to, used with, communicating with or linking a previous or new device model to the focal device.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Coding relation: The type indicates the relationship of the related device to the device instance
    :param CodeableReference relatedDevice: A reference to the linked device
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "relation": {"class_name": "Coding", "is_contained": False},
        "relatedDevice": {"class_name": "CodeableReference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        relation: "Coding" = None,
        relatedDevice: "CodeableReference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.relation = relation
        self.relatedDevice = relatedDevice

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Material(FhirBaseModel):
    """A substance used to create the material(s) of which the device is made.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept substance: A relevant substance that the device contains, may contain, or is made of
    :param bool alternate: Indicates an alternative material of the device
    :param bool allergenicIndicator: Whether the substance is a known or suspected allergen
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "substance": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        substance: "CodeableConcept" = None,
        alternate: "bool" = None,
        allergenicIndicator: "bool" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.substance = substance
        self.alternate = alternate
        self.allergenicIndicator = allergenicIndicator

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Guideline(FhirBaseModel):
    """Information aimed at providing directions for the usage of this model of device.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param UsageContext useContext: The circumstances that form the setting for using the device
    :param str usageInstruction: Detailed written and visual directions for the user on how to use the device
    :param RelatedArtifact relatedArtifact: A source of information or reference for this guideline
    :param CodeableConcept indication: A clinical condition for which the device was designed to be used
    :param CodeableConcept contraindication: A specific situation when a device should not be used because it may cause harm
    :param CodeableConcept warning: Specific hazard alert information that a user needs to know before using the device
    :param str intendedUse: A description of the general purpose or medical use of the device or its function
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
        "relatedArtifact": {"class_name": "RelatedArtifact", "is_contained": False},
        "indication": {"class_name": "CodeableConcept", "is_contained": False},
        "contraindication": {"class_name": "CodeableConcept", "is_contained": False},
        "warning": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        useContext: list["UsageContext"] = None,
        usageInstruction: "str" = None,
        relatedArtifact: list["RelatedArtifact"] = None,
        indication: list["CodeableConcept"] = None,
        contraindication: list["CodeableConcept"] = None,
        warning: list["CodeableConcept"] = None,
        intendedUse: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.useContext = useContext or []
        self.usageInstruction = usageInstruction
        self.relatedArtifact = relatedArtifact or []
        self.indication = indication or []
        self.contraindication = contraindication or []
        self.warning = warning or []
        self.intendedUse = intendedUse

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class CorrectiveAction(FhirBaseModel):
    """Tracking of latest field safety corrective action.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param bool recall: Whether the corrective action was a recall
    :param str scope: model | lot-numbers | serial-numbers
    :param Period period: Start and end dates of the  corrective action
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        recall: "bool" = None,
        scope: "str" = None,
        period: "Period" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.recall = recall
        self.scope = scope
        self.period = period

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class ChargeItem(FhirBaseModel):
    """Billing code or reference associated with the device.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableReference chargeItemCode: The code or reference for the charge item
    :param Quantity count: Coefficient applicable to the billing code
    :param Period effectivePeriod: A specific time period in which this charge item applies
    :param UsageContext useContext: The context to which this charge item applies
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "chargeItemCode": {"class_name": "CodeableReference", "is_contained": False},
        "count": {"class_name": "Quantity", "is_contained": False},
        "effectivePeriod": {"class_name": "Period", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        chargeItemCode: "CodeableReference" = None,
        count: "Quantity" = None,
        effectivePeriod: "Period" = None,
        useContext: list["UsageContext"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.chargeItemCode = chargeItemCode
        self.count = count
        self.effectivePeriod = effectivePeriod
        self.useContext = useContext or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class DeviceDefinition(DomainResource):
    """This is a specialized resource that defines the characteristics and capabilities of a device.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param str description: Additional information to describe the device
    :param Identifier identifier: Instance identifier
    :param UdiDeviceIdentifier udiDeviceIdentifier: Unique Device Identifier (UDI) Barcode string
    :param RegulatoryIdentifier regulatoryIdentifier: Regulatory identifier(s) associated with this device
    :param str partNumber: The part number or catalog number of the device
    :param Reference manufacturer: Name of device manufacturer
    :param DeviceName deviceName: The name or names of the device as given by the manufacturer
    :param str modelNumber: The catalog or model number for the device for example as defined by the manufacturer
    :param Classification classification: What kind of device or device system this is
    :param ConformsTo conformsTo: Identifies the standards, specifications, or formal guidances for the capabilities supported by the device
    :param HasPart hasPart: A device, part of the current one
    :param Packaging packaging: Information about the packaging of the device, i.e. how the device is packaged
    :param Version version: The version of the device or software
    :param CodeableConcept safety: Safety characteristics of the device
    :param ProductShelfLife shelfLifeStorage: Shelf Life and storage information
    :param CodeableConcept languageCode: Language code for the human-readable text strings produced by the device (all supported)
    :param Property property: Inherent, essentially fixed, characteristics of this kind of device, e.g., time properties, size, etc
    :param Reference owner: Organization responsible for device
    :param ContactPoint contact: Details for human/organization for support
    :param Link link: An associated device, attached to, used with, communicating with or linking a previous or new device model to the focal device
    :param Annotation note: Device notes and comments
    :param Material material: A substance used to create the material(s) of which the device is made
    :param str productionIdentifierInUDI: lot-number | manufactured-date | serial-number | expiration-date | biological-source | software-version
    :param Guideline guideline: Information aimed at providing directions for the usage of this model of device
    :param CorrectiveAction correctiveAction: Tracking of latest field safety corrective action
    :param ChargeItem chargeItem: Billing code or reference associated with the device
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "udiDeviceIdentifier": {
            "class_name": "UdiDeviceIdentifier",
            "is_contained": True,
        },
        "regulatoryIdentifier": {
            "class_name": "RegulatoryIdentifier",
            "is_contained": True,
        },
        "manufacturer": {"class_name": "Reference", "is_contained": False},
        "deviceName": {"class_name": "DeviceName", "is_contained": True},
        "classification": {"class_name": "Classification", "is_contained": True},
        "conformsTo": {"class_name": "ConformsTo", "is_contained": True},
        "hasPart": {"class_name": "HasPart", "is_contained": True},
        "packaging": {"class_name": "Packaging", "is_contained": True},
        "version": {"class_name": "Version", "is_contained": True},
        "safety": {"class_name": "CodeableConcept", "is_contained": False},
        "shelfLifeStorage": {"class_name": "ProductShelfLife", "is_contained": False},
        "languageCode": {"class_name": "CodeableConcept", "is_contained": False},
        "property": {"class_name": "Property", "is_contained": True},
        "owner": {"class_name": "Reference", "is_contained": False},
        "contact": {"class_name": "ContactPoint", "is_contained": False},
        "link": {"class_name": "Link", "is_contained": True},
        "note": {"class_name": "Annotation", "is_contained": False},
        "material": {"class_name": "Material", "is_contained": True},
        "guideline": {"class_name": "Guideline", "is_contained": True},
        "correctiveAction": {"class_name": "CorrectiveAction", "is_contained": True},
        "chargeItem": {"class_name": "ChargeItem", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        description: "str" = None,
        identifier: list["Identifier"] = None,
        udiDeviceIdentifier: list["UdiDeviceIdentifier"] = None,
        regulatoryIdentifier: list["RegulatoryIdentifier"] = None,
        partNumber: "str" = None,
        manufacturer: "Reference" = None,
        deviceName: list["DeviceName"] = None,
        modelNumber: "str" = None,
        classification: list["Classification"] = None,
        conformsTo: list["ConformsTo"] = None,
        hasPart: list["HasPart"] = None,
        packaging: list["Packaging"] = None,
        version: list["Version"] = None,
        safety: list["CodeableConcept"] = None,
        shelfLifeStorage: list["ProductShelfLife"] = None,
        languageCode: list["CodeableConcept"] = None,
        property: list["Property"] = None,
        owner: "Reference" = None,
        contact: list["ContactPoint"] = None,
        link: list["Link"] = None,
        note: list["Annotation"] = None,
        material: list["Material"] = None,
        productionIdentifierInUDI: list["str"] = None,
        guideline: "Guideline" = None,
        correctiveAction: "CorrectiveAction" = None,
        chargeItem: list["ChargeItem"] = None,
    ):

        self.resourceType = "DeviceDefinition"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.description = description
        self.identifier = identifier or []
        self.udiDeviceIdentifier = udiDeviceIdentifier or []
        self.regulatoryIdentifier = regulatoryIdentifier or []
        self.partNumber = partNumber
        self.manufacturer = manufacturer
        self.deviceName = deviceName or []
        self.modelNumber = modelNumber
        self.classification = classification or []
        self.conformsTo = conformsTo or []
        self.hasPart = hasPart or []
        self.packaging = packaging or []
        self.version = version or []
        self.safety = safety or []
        self.shelfLifeStorage = shelfLifeStorage or []
        self.languageCode = languageCode or []
        self.property = property or []
        self.owner = owner
        self.contact = contact or []
        self.link = link or []
        self.note = note or []
        self.material = material or []
        self.productionIdentifierInUDI = productionIdentifierInUDI or []
        self.guideline = guideline
        self.correctiveAction = correctiveAction
        self.chargeItem = chargeItem or []

    @classmethod
    def from_dict(cls, data: dict) -> "DeviceDefinition":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "DeviceDefinition":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for AppointmentResponse. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.DomainResource import *
from fhirmodels.R4.Extension import *
from fhirmodels.R4.Identifier import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Narrative import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.Resource import *


class AppointmentResponse(DomainResource):
    """A reply to an appointment request for a patient and/or practitioner(s), such as a confirmation or rejection.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: External Ids for this item
    :param Reference appointment: Appointment this response relates to
    :param str start: Time from appointment, or requested new start time
    :param str end: Time from appointment, or requested new end time
    :param CodeableConcept participantType: Role of participant in the appointment
    :param Reference actor: Person, Location, HealthcareService, or Device
    :param str participantStatus: accepted | declined | tentative | needs-action
    :param str comment: Additional comments
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "appointment": {"class_name": "Reference", "is_contained": False},
        "participantType": {"class_name": "CodeableConcept", "is_contained": False},
        "actor": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        appointment: "Reference" = None,
        start: "str" = None,
        end: "str" = None,
        participantType: list["CodeableConcept"] = None,
        actor: "Reference" = None,
        participantStatus: "str" = None,
        comment: "str" = None,
    ):

        self.resourceType = "AppointmentResponse"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.appointment = appointment
        self.start = start
        self.end = end
        self.participantType = participantType or []
        self.actor = actor
        self.participantStatus = participantStatus
        self.comment = comment

    @classmethod
    def from_dict(cls, data: dict) -> "AppointmentResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "AppointmentResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



import logging
import os
import pathlib
from datetime import datetime
from enum import Enum
from importlib import import_module
from urllib.parse import urlparse

from jinja2 import Environment, FileSystemLoader

from fhirmodels import constants as _c
from fhirmodels import fhir_package
from fhirmodels import utils as utils

logger = logging.getLogger(__name__)


class StructureDefinitionKinds(Enum):
    PRIMITIVE_TYPE = "primitive-type"
    COMPLEX_TYPE = "complex-type"
    RESOURCE = "resource"
    LOGICAL = "logical"


class FhirBaseModel:
    """Base class for all generated models."""

    property_class_info = {}

    def __init__(self):
        pass

    @classmethod
    def from_dict(cls, data: dict):
        """Create a model instance from a dict. The instance is recursively
        created by importing the classes for complex FHIR types."""
        instance = cls()
        models_path = utils.remove_n_parts_from_end(cls.__module__, 1)

        def handle_dict_value(key: str, value: dict):
            # Capitalize the class name if it's not
            if key not in cls.property_class_info:
                setattr(instance, key, value)
                return
            class_name = instance.property_class_info[key]["class_name"]
            is_contained = cls.property_class_info[key]["is_contained"]
            if is_contained:
                module = import_module(cls.__module__)
                model_class = getattr(module, class_name)
            else:
                import_path = f"{models_path}.{class_name}"
                module = import_module(import_path)
                model_class = getattr(module, class_name)
            nested_instance = model_class.from_dict(value)
            return nested_instance

        for key, value in data.items():
            if isinstance(value, dict):  # complex type
                nested_instance = handle_dict_value(key, value)
                setattr(instance, key, nested_instance)
            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict):  # complex type
                        # mutating mutable object no need to reassign
                        if key not in cls.property_class_info:
                            if hasattr(instance, key):
                                getattr(instance, key).append(item)
                            else:
                                setattr(instance, key, [item])
                            continue
                        nested_instance = handle_dict_value(key, item)
                        getattr(instance, key).append(nested_instance)
                    else:  # primitive type
                        getattr(instance, key).append(item)
            else:  # primitive type
                setattr(instance, key, value)

        return instance

    @classmethod
    def from_obj(cls, data):
        """Creates a model instance from an object or a dictionary."""
        if isinstance(data, dict):
            return cls.from_dict(data)
        elif hasattr(data, "as_dict") and callable(data.as_dict):
            return cls.from_dict(data.as_dict())
        else:
            raise ValueError(
                "Invalid input object. Must be a dictionary or an object with an 'as_dict' method."
            )

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.as_dict()})"

    def as_dict(self):
        # from https://stackoverflow.com/questions/7963762/what-is-the-most-economical-way-to-convert-nested-python-objects-to-dictionaries
        def serialize(obj):
            if isinstance(obj, FhirBaseModel):
                return obj.as_dict()
            elif isinstance(obj, (list, tuple)):
                return [serialize(item) for item in obj]
            elif isinstance(obj, dict):
                return {
                    key: serialize(value)
                    for key, value in obj.items()
                    if value is not None and value != []
                }
            else:
                return obj  # Handle basic types

        return {
            key: serialize(value)
            for key, value in self.__dict__.items()
            if value is not None and value != []
        }


class GeneratorElement:
    """Wrapper around a ElementDefinition with a parsed name and type"""

    def __init__(
        self,
        element_definition: dict,
        name: str,
        type: str,
        children: list["GeneratorElement"] = None,
        is_contained: bool = False,
    ):
        children = children or []
        self._element = element_definition
        # the element is defined by referencng another element this means we dont have to resolve
        # created a new class if its a contained element (its created by the referenced element)
        self.is_referencing = (
            element_definition.get("contentReference", None) is not None
        )
        self.id = element_definition["id"]
        self.type = type
        self.path: str = element_definition["path"]
        self.name = _escape_keyword(name) if name in _c.PYTHON_KEYWORDS else name
        self.min, self.max = self._get_cardinalities()
        self.is_primitive = utils.is_primitive_type(self.type)
        self.description = element_definition.get("definition", "")
        self.short = element_definition.get("short", "")
        self.is_contained = is_contained
        self.python_type = (
            utils.get_python_type_for_primitive(self.type)
            if self.is_primitive
            else None
        )
        self.base_class, self.base_import_string = _get_base_class(
            kind=StructureDefinitionKinds.COMPLEX_TYPE.value, type=self.type
        )
        (
            self.contained_children,
            self.defined_children,
        ) = _filter_contained_elements(children)

    @property
    def is_array(self):
        return self.max > 1

    @property
    def type_string(self):
        """Type used for the jinja template"""
        if self.is_primitive:  # primitive with python type
            return self.python_type.__name__
        else:
            return f"{self.type}"

    def __eq__(self, __value: "GeneratorElement") -> bool:
        """Compares two GeneratorElements by their path, type, name and id.
        Primarily used to filter duplicate elements."""
        return (
            self.path == __value.path
            and self.type == __value.type
            and self.name == __value.name
            and self.id == __value.id
        )

    def _get_cardinalities(self) -> tuple[float, float]:
        """Returns the min and max cardinality of the ElementDefinition."""
        min = self._element.get("min", None)
        max = self._element.get("max", None)
        if max == "*":
            max = float("inf")
        return float(min), float(max)


class GeneratorStructureDefinition:
    """Represents a FHIR StructureDefinition that is used to generate downsream objects."""

    def __init__(self, structure_definition: dict):
        self._structure_definition = structure_definition
        self._element_definitions = structure_definition.get("snapshot", {}).get(
            "element", []
        )
        self.generator_elements = self._initialize_elements(self._element_definitions)
        (
            self.contained_elements,
            self.defined_elements,
        ) = _filter_contained_elements(self.generator_elements)
        self.kind = self._structure_definition["kind"]
        self.is_primitive = self.kind == StructureDefinitionKinds.PRIMITIVE_TYPE.value
        self.id = self._structure_definition["id"]
        self.has_snapshot = self._structure_definition.get("snapshot", None) is not None
        self.description = self._structure_definition.get("description", "")
        self.short = self._structure_definition.get("short", "")
        self.type = self._structure_definition["type"]
        self.is_resource = (
            self._structure_definition.get("baseDefinition")
            == _c.DOMAIN_RESOURCE_STRUC_DEF_URL
            or self.id == "Base"
        )
        self.base_class, self.base_import_string = _get_base_class(
            kind=self.kind, type=self.type
        )
        # TODO: For now structure definitions with same id and type are considered base classes, but this is might always true
        self.is_base = self.id == self.type

    @property
    def dependencies(self):
        """Unique list of dependencies for this StructureDefinition. Used to generate
        the import statements for the generated class."""
        complex_types = [
            e.type
            for e in self.generator_elements
            if not e.is_primitive
            and not e.is_referencing  # handled by the referenced element
        ]
        # avoid circles
        complex_types = [t for t in complex_types if t != self.type]
        return list(set(complex_types))

    def _initialize_elements(self, element_definitions) -> list[GeneratorElement]:
        """Parse elements from the StructureDefinition to explicitly resolve
        the element name and type."""
        parsed_elements = []
        for element_definition in element_definitions:
            if utils.is_root_path(element_definition["id"]):
                continue
            parsed_elements.extend(self._parse_element(element_definition))
        return parsed_elements

    def _parse_element(self, element_definition: dict) -> list[GeneratorElement]:
        """Parse name and type of a single ElementDefinition. Choice elements are
        expanded into multiple elements with the choice string '[x]' being replaced
        by the type. Elements where the type can not be resolved are skipped."""
        result = []
        types = element_definition.get("type", [])
        element_name = utils.get_nth_part(element_definition["id"], -1)
        choice_element_name = None
        is_choice = utils.is_choice_path(element_definition["path"])

        # Try to resolve missing types once
        if not types:
            element = self._resolve_missing_type(element_definition)
            if element:
                # Merge element and keep valies from element_definition
                # More info: https://stackoverflow.com/questions/38987/how-do-i-merge-two-dictionaries-in-a-single-expression-in-python
                element_definition = element | element_definition
                types = element_definition.get("type", [])

        for type_info in types:
            type_code = type_info["code"]

            # Handle choice elements
            if is_choice:
                choice_element_name = utils.replace_choice_string(
                    path=element_name, replace=_capitalize_first_letter(type_code)
                )

            element = GeneratorElement(
                element_definition=element_definition,
                name=choice_element_name or element_name,
                type=type_code,
            )
            result.append(element)
        return result

    def _resolve_missing_type(self, element_definition: dict):
        """Try to find an ElementDefinition with resolved type."""
        if element_definition.get("contentReference"):
            # try to resolve the type by resolving the content reference
            return self._resolve_content_reference(element_definition)
        return None

    def _resolve_content_reference(self, element_definition: dict):
        """Returns an ElementDefinition by resolving the content reference.
        More info: https://build.fhir.org/elementdefinition-definitions.html#ElementDefinition.contentReference
        """
        referenced_element = None
        content_reference_uri = element_definition.get("contentReference", None)
        parsed_uri = urlparse(content_reference_uri)
        if parsed_uri.fragment:
            # try to resolve the referenced element within local StructureDefinition
            referenced_element = self._get_original_element_by_id(parsed_uri.fragment)
        # TODO try to resolve the referenced element from another StructureDefinition
        return referenced_element

    def _get_original_element_by_id(self, id: str):
        """Get an ElementDefinition from the original input StructureDefinition."""
        elements = self._structure_definition["snapshot"]["element"]
        for element in elements:
            if element["id"] == id:
                return element
        return None


def _filter_contained_elements(
    elements: list[GeneratorElement],
) -> tuple[list[GeneratorElement], list[GeneratorElement]]:
    """Filters the given elements for Elements that are defined within the resource.
    They are treated separately in the template because the type is not knonw before
    parsing the StructureDefinition."""
    contained_generator_elements = []  # Elements of type BackboneElement or Element
    defined_generator_elements = []
    seen_elements = []
    for element in elements:
        if element in seen_elements:  # Can't use path because of choice elements
            continue
        seen_elements.append(element)  # Add the element to the set
        if element.type == "BackboneElement" or element.type == "Element":
            if element.is_referencing:
                # The element is defined by referencing another element, the class
                # for the contained element is created by the referenced element
                element.type = _capitalize_first_letter(element.name)
                element.is_contained = True
                defined_generator_elements.append(element)
                continue
            children = [
                e
                for e in elements
                if e.path.startswith(element.path)
                and e.path
                != element.path  # All children of the contained element without the element itself
            ]
            seen_elements.extend(children)  # Add children to the seen elements
            contained_element = GeneratorElement(
                element_definition=element._element,
                name=element.name,
                type=_capitalize_first_letter(element.name),
                children=children,
                is_contained=True,
            )
            contained_generator_elements.append(contained_element)
            # Contained Element is now defined (type is known, itself), its children are not
            defined_generator_elements.append(contained_element)
        else:
            defined_generator_elements.append(element)
    return contained_generator_elements, defined_generator_elements


def _get_base_class(kind: str, type: str) -> tuple[str, str]:
    """Returns the base class according to the fhir spec
    and its import string for a StructureDefinition.
    If the base class is generated by the generator, the import string is None."""

    class_str = FhirBaseModel.__name__
    import_str = _import_string_for_path(FhirBaseModel.__module__)

    if kind == StructureDefinitionKinds.RESOURCE.value:
        if type == "Resource":
            return class_str, import_str
        if type == "DomainResource":
            return "Resource", None
        else:
            return "DomainResource", None
    else:
        return class_str, import_str


def _escape_keyword(name: str) -> str:
    """Escapes a Python keyword."""
    return f"_{name}"


def _import_string_for_path(path: str):
    """Returns the import string for the given path relative
    to src directory."""
    if "src/" in path:
        path = path.split("src/")[-1]
    if ".py" in path:
        path = path.split(".py")[0]
    import_str = path.replace("/", ".")
    return import_str


def _capitalize_first_letter(string: str) -> str:
    """Capitalize the first letter of a string. Used parse
    choice types."""
    return string[0].upper() + string[1:]


class ModelGenerator:
    """Generates model classes from FHIR Packages."""

    def __init__(
        self,
        package: fhir_package.FhirPackage,
        template_dir: str = None,
        output_dir: str = None,
        template_names: list[str] = None,
    ):
        self._package = package

        # specify output dir for generated model classes
        self._output_dir = output_dir or _c.DEFAULT_OUTPUT_DIR
        self._output_dir = get_full_path_to_dir(
            os.path.join(self._output_dir, self._package.name)
        )
        open(os.path.join(self._output_dir, "__init__.py"), "a").close()

        # specify template dir for jinja2 templates
        self._template_dir = template_dir or _c.TEMPLATE_DIR
        self._template_dir = get_full_path_to_dir(self._template_dir)

        self._structure_def_model_template = _c.STRUC_DEF_TEMPLATE_NAME
        self._init_model_template = _c.MODEL_TEMPLATE_NAME
        self.generator_struc_defs = [
            GeneratorStructureDefinition(sd)
            for sd in self._package.structure_definitions
        ]

    def generate_model_classes(self):
        """Create the python classes for the FHIR package."""
        logger.info("Generating model classes for %s", self._package.name)
        logger.info("Output dir: %s", self._output_dir)
        self._generate_structure_definition_classes()
        self._generate_model_init()

    def _generate_structure_definition_classes(self):
        """Generate the python classes for the StructureDefinitions in the FHIR package."""
        env = Environment(loader=FileSystemLoader(searchpath=self._template_dir))
        template = env.get_template(self._structure_def_model_template)
        for structure_definition in self.generator_struc_defs:
            if (
                not structure_definition.has_snapshot  # no elemnns present
                or structure_definition.is_primitive
                or not structure_definition.is_base  # TODO: add support for Extensions and non-base classes
            ):
                continue
            model_code = template.render(
                structure_definition=structure_definition,
                base_class="",
                dir_name=self._package.name,
                time=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                source_file=__file__,
            )
            output_file = f"{self._output_dir}/{structure_definition.type}.py"
            with open(output_file, "w") as f:
                f.write(model_code)

    def _generate_model_init(self):
        env = Environment(loader=FileSystemLoader(searchpath=self._template_dir))
        template = env.get_template(self._init_model_template)
        klasses = []
        for structure_definition in self.generator_struc_defs:
            if (
                not structure_definition.has_snapshot  # no elemnns present
                or structure_definition.is_primitive
                or not structure_definition.is_base  # TODO: add support for Extensions and non-base classes
            ):
                continue
            klasses.append(structure_definition.type)
        model_init_code = template.render(
            time=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            source_file=__file__,
            klasses=klasses,
        )
        output_file = f"{self._output_dir}/__init__.py"
        with open(output_file, "w") as f:
            f.write(model_init_code)


def get_full_path_to_dir(dir: str):
    full_path = pathlib.Path(os.path.dirname(os.path.realpath(__file__))) / dir
    if not os.path.exists(full_path):
        os.makedirs(full_path)
    return full_path


if __name__ == "__main__":
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    )
    package_loader = fhir_package.FhirPackageLoader()
    package = package_loader.load_from_version(fhir_version="R4")
    generator = ModelGenerator(package=package)
    generator.generate_model_classes()



"""
Generated class for SubstanceSpecification. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.R4.Attachment import *
from fhirmodels.R4.BackboneElement import *
from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.DomainResource import *
from fhirmodels.R4.Extension import *
from fhirmodels.R4.Identifier import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Narrative import *
from fhirmodels.R4.Quantity import *
from fhirmodels.R4.Range import *
from fhirmodels.R4.Ratio import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.Resource import *


class Moiety(FhirBaseModel):
    """Moiety, for structural modifications.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept role: Role that the moiety is playing
    :param Identifier identifier: Identifier by which this moiety substance is known
    :param str name: Textual name for this moiety substance
    :param CodeableConcept stereochemistry: Stereochemistry type
    :param CodeableConcept opticalActivity: Optical activity type
    :param str molecularFormula: Molecular formula
    :param Quantity amountQuantity: Quantitative value for this moiety
    :param str amountString: Quantitative value for this moiety
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "stereochemistry": {"class_name": "CodeableConcept", "is_contained": False},
        "opticalActivity": {"class_name": "CodeableConcept", "is_contained": False},
        "amountQuantity": {"class_name": "Quantity", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        role: "CodeableConcept" = None,
        identifier: "Identifier" = None,
        name: "str" = None,
        stereochemistry: "CodeableConcept" = None,
        opticalActivity: "CodeableConcept" = None,
        molecularFormula: "str" = None,
        amountQuantity: "Quantity" = None,
        amountString: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.role = role
        self.identifier = identifier
        self.name = name
        self.stereochemistry = stereochemistry
        self.opticalActivity = opticalActivity
        self.molecularFormula = molecularFormula
        self.amountQuantity = amountQuantity
        self.amountString = amountString

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Property(FhirBaseModel):
    """General specifications for this substance, including how it is related to other substances.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept category: A category for this property, e.g. Physical, Chemical, Enzymatic
    :param CodeableConcept code: Property type e.g. viscosity, pH, isoelectric point
    :param str parameters: Parameters that were used in the measurement of a property (e.g. for viscosity: measured at 20C with a pH of 7.1)
    :param Reference definingSubstanceReference: A substance upon which a defining property depends (e.g. for solubility: in water, in alcohol)
    :param CodeableConcept definingSubstanceCodeableConcept: A substance upon which a defining property depends (e.g. for solubility: in water, in alcohol)
    :param Quantity amountQuantity: Quantitative value for this property
    :param str amountString: Quantitative value for this property
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "category": {"class_name": "CodeableConcept", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "definingSubstanceReference": {
            "class_name": "Reference",
            "is_contained": False,
        },
        "definingSubstanceCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "amountQuantity": {"class_name": "Quantity", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        category: "CodeableConcept" = None,
        code: "CodeableConcept" = None,
        parameters: "str" = None,
        definingSubstanceReference: "Reference" = None,
        definingSubstanceCodeableConcept: "CodeableConcept" = None,
        amountQuantity: "Quantity" = None,
        amountString: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.category = category
        self.code = code
        self.parameters = parameters
        self.definingSubstanceReference = definingSubstanceReference
        self.definingSubstanceCodeableConcept = definingSubstanceCodeableConcept
        self.amountQuantity = amountQuantity
        self.amountString = amountString

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class MolecularWeight(FhirBaseModel):
    """The molecular weight or weight range (for proteins, polymers or nucleic acids).:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept method: The method by which the molecular weight was determined
    :param CodeableConcept type: Type of molecular weight such as exact, average (also known as. number average), weight average
    :param Quantity amount: Used to capture quantitative values for a variety of elements. If only limits are given, the arithmetic mean would be the average. If only a single definite value for a given element is given, it would be captured in this field
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "method": {"class_name": "CodeableConcept", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "amount": {"class_name": "Quantity", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        method: "CodeableConcept" = None,
        type: "CodeableConcept" = None,
        amount: "Quantity" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.method = method
        self.type = type
        self.amount = amount

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Isotope(FhirBaseModel):
    """Applicable for single substances that contain a radionuclide or a non-natural isotopic ratio.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Identifier identifier: Substance identifier for each non-natural or radioisotope
    :param CodeableConcept name: Substance name for each non-natural or radioisotope
    :param CodeableConcept substitution: The type of isotopic substitution present in a single substance
    :param Quantity halfLife: Half life - for a non-natural nuclide
    :param MolecularWeight molecularWeight: The molecular weight or weight range (for proteins, polymers or nucleic acids)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "name": {"class_name": "CodeableConcept", "is_contained": False},
        "substitution": {"class_name": "CodeableConcept", "is_contained": False},
        "halfLife": {"class_name": "Quantity", "is_contained": False},
        "molecularWeight": {"class_name": "MolecularWeight", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: "Identifier" = None,
        name: "CodeableConcept" = None,
        substitution: "CodeableConcept" = None,
        halfLife: "Quantity" = None,
        molecularWeight: "MolecularWeight" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier
        self.name = name
        self.substitution = substitution
        self.halfLife = halfLife
        self.molecularWeight = molecularWeight

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Representation(FhirBaseModel):
    """Molecular structural representation.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: The type of structure (e.g. Full, Partial, Representative)
    :param str representation: The structural representation as text string in a format e.g. InChI, SMILES, MOLFILE, CDX
    :param Attachment attachment: An attached file with the structural representation
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "attachment": {"class_name": "Attachment", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        representation: "str" = None,
        attachment: "Attachment" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.representation = representation
        self.attachment = attachment

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Structure(FhirBaseModel):
    """Structural information.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept stereochemistry: Stereochemistry type
    :param CodeableConcept opticalActivity: Optical activity type
    :param str molecularFormula: Molecular formula
    :param str molecularFormulaByMoiety: Specified per moiety according to the Hill system, i.e. first C, then H, then alphabetical, each moiety separated by a dot
    :param Isotope isotope: Applicable for single substances that contain a radionuclide or a non-natural isotopic ratio
    :param MolecularWeight molecularWeight: The molecular weight or weight range (for proteins, polymers or nucleic acids)
    :param Reference source: Supporting literature
    :param Representation representation: Molecular structural representation
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "stereochemistry": {"class_name": "CodeableConcept", "is_contained": False},
        "opticalActivity": {"class_name": "CodeableConcept", "is_contained": False},
        "isotope": {"class_name": "Isotope", "is_contained": True},
        "molecularWeight": {"class_name": "MolecularWeight", "is_contained": True},
        "source": {"class_name": "Reference", "is_contained": False},
        "representation": {"class_name": "Representation", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        stereochemistry: "CodeableConcept" = None,
        opticalActivity: "CodeableConcept" = None,
        molecularFormula: "str" = None,
        molecularFormulaByMoiety: "str" = None,
        isotope: list["Isotope"] = None,
        molecularWeight: "MolecularWeight" = None,
        source: list["Reference"] = None,
        representation: list["Representation"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.stereochemistry = stereochemistry
        self.opticalActivity = opticalActivity
        self.molecularFormula = molecularFormula
        self.molecularFormulaByMoiety = molecularFormulaByMoiety
        self.isotope = isotope or []
        self.molecularWeight = molecularWeight
        self.source = source or []
        self.representation = representation or []

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Code(FhirBaseModel):
    """Codes associated with the substance.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept code: The specific code
    :param CodeableConcept status: Status of the code assignment
    :param str statusDate: The date at which the code status is changed as part of the terminology maintenance
    :param str comment: Any comment can be provided in this field, if necessary
    :param Reference source: Supporting literature
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "status": {"class_name": "CodeableConcept", "is_contained": False},
        "source": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "CodeableConcept" = None,
        status: "CodeableConcept" = None,
        statusDate: "str" = None,
        comment: "str" = None,
        source: list["Reference"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.status = status
        self.statusDate = statusDate
        self.comment = comment
        self.source = source or []

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Official(FhirBaseModel):
    """Details of the official nature of this name.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept authority: Which authority uses this official name
    :param CodeableConcept status: The status of the official name
    :param str date: Date of official name change
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "authority": {"class_name": "CodeableConcept", "is_contained": False},
        "status": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        authority: "CodeableConcept" = None,
        status: "CodeableConcept" = None,
        date: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.authority = authority
        self.status = status
        self.date = date

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Name(FhirBaseModel):
    """Names applicable to this substance.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str name: The actual name
    :param CodeableConcept type: Name type
    :param CodeableConcept status: The status of the name
    :param bool preferred: If this is the preferred name for this substance
    :param CodeableConcept language: Language of the name
    :param CodeableConcept domain: The use context of this name for example if there is a different name a drug active ingredient as opposed to a food colour additive
    :param CodeableConcept jurisdiction: The jurisdiction where this name applies
    :param Synonym synonym: A synonym of this name
    :param Translation translation: A translation for this name
    :param Official official: Details of the official nature of this name
    :param Reference source: Supporting literature
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "status": {"class_name": "CodeableConcept", "is_contained": False},
        "language": {"class_name": "CodeableConcept", "is_contained": False},
        "domain": {"class_name": "CodeableConcept", "is_contained": False},
        "jurisdiction": {"class_name": "CodeableConcept", "is_contained": False},
        "synonym": {"class_name": "Synonym", "is_contained": True},
        "translation": {"class_name": "Translation", "is_contained": True},
        "official": {"class_name": "Official", "is_contained": True},
        "source": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        name: "str" = None,
        type: "CodeableConcept" = None,
        status: "CodeableConcept" = None,
        preferred: "bool" = None,
        language: list["CodeableConcept"] = None,
        domain: list["CodeableConcept"] = None,
        jurisdiction: list["CodeableConcept"] = None,
        synonym: list["Synonym"] = None,
        translation: list["Translation"] = None,
        official: list["Official"] = None,
        source: list["Reference"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.name = name
        self.type = type
        self.status = status
        self.preferred = preferred
        self.language = language or []
        self.domain = domain or []
        self.jurisdiction = jurisdiction or []
        self.synonym = synonym or []
        self.translation = translation or []
        self.official = official or []
        self.source = source or []

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Relationship(FhirBaseModel):
    """A link between this substance and another, with details of the relationship.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference substanceReference: A pointer to another substance, as a resource or just a representational code
    :param CodeableConcept substanceCodeableConcept: A pointer to another substance, as a resource or just a representational code
    :param CodeableConcept relationship: For example "salt to parent", "active moiety", "starting material"
    :param bool isDefining: For example where an enzyme strongly bonds with a particular substance, this is a defining relationship for that enzyme, out of several possible substance relationships
    :param Quantity amountQuantity: A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other
    :param Range amountRange: A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other
    :param Ratio amountRatio: A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other
    :param str amountString: A numeric factor for the relationship, for instance to express that the salt of a substance has some percentage of the active substance in relation to some other
    :param Ratio amountRatioLowLimit: For use when the numeric
    :param CodeableConcept amountType: An operator for the amount, for example "average", "approximately", "less than"
    :param Reference source: Supporting literature
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "substanceReference": {"class_name": "Reference", "is_contained": False},
        "substanceCodeableConcept": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
        "relationship": {"class_name": "CodeableConcept", "is_contained": False},
        "amountQuantity": {"class_name": "Quantity", "is_contained": False},
        "amountRange": {"class_name": "Range", "is_contained": False},
        "amountRatio": {"class_name": "Ratio", "is_contained": False},
        "amountRatioLowLimit": {"class_name": "Ratio", "is_contained": False},
        "amountType": {"class_name": "CodeableConcept", "is_contained": False},
        "source": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        substanceReference: "Reference" = None,
        substanceCodeableConcept: "CodeableConcept" = None,
        relationship: "CodeableConcept" = None,
        isDefining: "bool" = None,
        amountQuantity: "Quantity" = None,
        amountRange: "Range" = None,
        amountRatio: "Ratio" = None,
        amountString: "str" = None,
        amountRatioLowLimit: "Ratio" = None,
        amountType: "CodeableConcept" = None,
        source: list["Reference"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.substanceReference = substanceReference
        self.substanceCodeableConcept = substanceCodeableConcept
        self.relationship = relationship
        self.isDefining = isDefining
        self.amountQuantity = amountQuantity
        self.amountRange = amountRange
        self.amountRatio = amountRatio
        self.amountString = amountString
        self.amountRatioLowLimit = amountRatioLowLimit
        self.amountType = amountType
        self.source = source or []

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class SubstanceSpecification(DomainResource):
    """The detailed description of a substance, typically at a level beyond what is used for prescribing.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Identifier by which this substance is known
    :param CodeableConcept type: High level categorization, e.g. polymer or nucleic acid
    :param CodeableConcept status: Status of substance within the catalogue e.g. approved
    :param CodeableConcept domain: If the substance applies to only human or veterinary use
    :param str description: Textual description of the substance
    :param Reference source: Supporting literature
    :param str comment: Textual comment about this record of a substance
    :param Moiety moiety: Moiety, for structural modifications
    :param Property property: General specifications for this substance, including how it is related to other substances
    :param Reference referenceInformation: General information detailing this substance
    :param Structure structure: Structural information
    :param Code code: Codes associated with the substance
    :param Name name: Names applicable to this substance
    :param MolecularWeight molecularWeight: The molecular weight or weight range (for proteins, polymers or nucleic acids)
    :param Relationship relationship: A link between this substance and another, with details of the relationship
    :param Reference nucleicAcid: Data items specific to nucleic acids
    :param Reference polymer: Data items specific to polymers
    :param Reference protein: Data items specific to proteins
    :param Reference sourceMaterial: Material or taxonomic/anatomical source for the substance
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "status": {"class_name": "CodeableConcept", "is_contained": False},
        "domain": {"class_name": "CodeableConcept", "is_contained": False},
        "source": {"class_name": "Reference", "is_contained": False},
        "moiety": {"class_name": "Moiety", "is_contained": True},
        "property": {"class_name": "Property", "is_contained": True},
        "referenceInformation": {"class_name": "Reference", "is_contained": False},
        "structure": {"class_name": "Structure", "is_contained": True},
        "code": {"class_name": "Code", "is_contained": True},
        "name": {"class_name": "Name", "is_contained": True},
        "molecularWeight": {"class_name": "MolecularWeight", "is_contained": True},
        "relationship": {"class_name": "Relationship", "is_contained": True},
        "nucleicAcid": {"class_name": "Reference", "is_contained": False},
        "polymer": {"class_name": "Reference", "is_contained": False},
        "protein": {"class_name": "Reference", "is_contained": False},
        "sourceMaterial": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: "Identifier" = None,
        type: "CodeableConcept" = None,
        status: "CodeableConcept" = None,
        domain: "CodeableConcept" = None,
        description: "str" = None,
        source: list["Reference"] = None,
        comment: "str" = None,
        moiety: list["Moiety"] = None,
        property: list["Property"] = None,
        referenceInformation: "Reference" = None,
        structure: "Structure" = None,
        code: list["Code"] = None,
        name: list["Name"] = None,
        molecularWeight: list["MolecularWeight"] = None,
        relationship: list["Relationship"] = None,
        nucleicAcid: "Reference" = None,
        polymer: "Reference" = None,
        protein: "Reference" = None,
        sourceMaterial: "Reference" = None,
    ):

        self.resourceType = "SubstanceSpecification"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier
        self.type = type
        self.status = status
        self.domain = domain
        self.description = description
        self.source = source or []
        self.comment = comment
        self.moiety = moiety or []
        self.property = property or []
        self.referenceInformation = referenceInformation
        self.structure = structure
        self.code = code or []
        self.name = name or []
        self.molecularWeight = molecularWeight or []
        self.relationship = relationship or []
        self.nucleicAcid = nucleicAcid
        self.polymer = polymer
        self.protein = protein
        self.sourceMaterial = sourceMaterial

    @classmethod
    def from_dict(cls, data: dict) -> "SubstanceSpecification":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "SubstanceSpecification":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()
